# Copyright (C) 2004 Laurent Sansonetti
#
# Tengen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Tengen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Tengen; see the file COPYING.  If not,
# write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

module Tengen
    class Action 
        attr_reader :vertex, :time, :kind
        
        MOVE, PASS, UNDO = 1, 2, 3
        
        def self.new_pass
            self.new(PASS)
        end

        def self.new_move(coord)
            self.new(MOVE, coord)
        end

        def self.new_undo
            self.new(UNDO)
        end

        def pass?
            kind == PASS
        end
    
        def move?
            kind == MOVE
        end

        def undo?
            kind == UNDO
        end
    
        def <=>(move)
            self.time <=> move.time
        end
        
        #######
        private
        #######

        def initialize(kind, arg=nil)
            @kind = kind
            if move? and arg 
                @vertex = arg.is_a?(GTP::Vertex) ? arg : GTP::Vertex.new(arg) 
            end
            @time = Time.now
        end
    end
end
